#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/* Global variables */
char *conf_log;
char *conf_dbtype;
char *conf_dbhost;
char *conf_db;
char *conf_dbuser;
char *conf_dbpass;
char *conf_dbport;
char *conf_dbsock;
char *conf_query;
char *conf_user, *conf_group;

/* Internal variables */
static char error[512];


char *sesoma_conf_strerror(void)
{
	return error;
}

static char *fix(char *s)
{
	int last;
	char *ret;

	if (s == NULL)
		return NULL;

	/* begin */
	ret = s;
	while ((*ret == ' ') || (*ret == '\t'))
		ret++;

	/* end */
	last = strlen(ret) - 1;
	while ((ret[last] == ' ') || (ret[last] == '\t') || (ret[last] == '\n') || (ret[last] == '\r')) {
		ret[last] = '\0';
		last--;
	}

	return ret;
}

static void change(char **to, char *from)
{
	if (*to != NULL)
		free(*to);
	*to = strdup(from);
}

int sesoma_conf_load(char *file)
{
	FILE *f;
	char *line, *l, *r, *eq;
	char buf[4096];
	int lineno, ok;

	strcpy(error, "");

	/* set defaults */
	conf_log = strdup("/var/log/sesoma.log");
	conf_dbtype = strdup("pgsql");
	conf_dbhost = NULL;
	conf_db = strdup("sesoma");
	conf_dbuser = NULL;
	conf_dbpass = NULL;
	conf_dbport = NULL;
	conf_dbsock = NULL;
	conf_query = strdup("SELECT r FROM %s WHERE l = '%s'");
	conf_user = strdup("nobody");
	conf_group = strdup("nogroup");

	f = fopen(file, "r");
	if (!f) {
		snprintf(error, sizeof(error),
			"Cannot load conf file. Using defaults. [%s]", strerror(errno));
		return -1;
	}

	lineno = 0;
	while (!feof(f)) {
		line = fgets(buf, sizeof(buf), f);
		if (!line)
			break;

		lineno++;

		if ((buf[0] == '#') || (buf[0] == '\n'))
			continue;

		eq = strchr(line, '=');
		if (!eq) {
			snprintf(error, sizeof(error),
				"Error parsing line %d! Missing value.", lineno);
			goto err;
		}

		*eq = '\0';
		l = buf;
		r = eq + 1;

		l = fix(l);
		r = fix(r);
		/*printf("l=[%s] r=[%s]\n", l, r);*/

		ok = 0;
		if (strcmp(l, "log") == 0) {
			change(&conf_log, r);
			ok = 1;
		}

		if (strcmp(l, "dbtype") == 0) {
			change(&conf_dbtype, r);
			ok = 1;
		}

		if (strcmp(l, "dbhost") == 0) {
			change(&conf_dbhost, r);
			ok = 1;
		}

		if (strcmp(l, "db") == 0) {
			change(&conf_db, r);
			ok = 1;
		}

		if (strcmp(l, "dbuser") == 0) {
			change(&conf_dbuser, r);
			ok = 1;
		}

		if (strcmp(l, "dbpass") == 0) {
			change(&conf_dbpass, r);
			ok = 1;
		}

		if (strcmp(l, "dbport") == 0) {
			change(&conf_dbport, r);
			ok = 1;
		}

		if (strcmp(l, "dbsock") == 0) {
			change(&conf_dbsock, r);
			ok = 1;
		}

		if (strcmp(l, "query") == 0) {
			change(&conf_query, r);
			ok = 1;
		}

		if (strcmp(l, "user") == 0) {
			change(&conf_user, r);
			ok = 1;
		}

		if (strcmp(l, "group") == 0) {
			change(&conf_group, r);
			ok = 1;
		}

		if (ok == 0) {
			snprintf(error, sizeof(error),
				"Error parsing line %d! Token not recognized.", lineno);
			goto err;
		}
	}

	return 0;

	err:
	fclose(f);
	return -1;

	return 0;
}
